/*******************************************************************************

	ZebraBookConv.c

	This program convert Zebra book to history text.

	Created:		June 29, 2008
	Author:			T.Ishii (t-ishii@js2.so-net.ne.jp)
	Author of Zebra: Gunnar Andersson (gunnar@radagast.se)

	- NOTE -
	Zebra is a famous reversi program released under the GPL Version 2.
	This program is made using Zebra source code.
	So, this program is also released under the GPL Version 2.
	See the file COPYING for more information.

	- Related Zebra source files -
	constant.h
	osfbook.h osfbook.c
	patterns.c
	myrandom.c

	- Copyright of myrandom.c -
	Copyright (C) 1995 DJ Delorie, see COPYING.DJ for details
	This file may have been modified by DJ Delorie (Jan 1995).  If so,
	these modifications are Coyright (C) 1993 DJ Delorie, 24 Kirsten Ave,
	Rochester NH, 03867-2954, USA.

	Copyright (c) 1983 Regents of the University of California.
	All rights reserved.

**********************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>

/* Book node definition */
typedef struct {
	int hash_val1;
	int hash_val2;
	short black_minimax_score;
	short white_minimax_score;
	short best_alternative_move;
	short alternative_score;
	unsigned short flags;
} BookNode;

/* Data structure definitions */
#define NODE_TABLE_SLACK	1000
#define EMPTY_HASH_SLOT		-1

/* Flag bits and shifts definitions */
#define NULL_MOVE			0
#define BLACK_TO_MOVE		1
#define WHITE_TO_MOVE		2
#define WLD_SOLVED			4
#define NOT_TRAVERSED		8
#define FULL_SOLVED			16
#define PRIVATE_NODE		32
#define DEPTH_SHIFT			10
#define USEDFLG				64		/* add only for ZebraBookConv */

/* Values denoting absent moves and scores */
#define NO_SCORE			9999
#define CONFIRMED_WIN		30000

/* Symbolic values for the possible contents of a square */
#define ILLEGAL				-1
#define BLACKSQ				0
#define EMPTY				1
#define WHITESQ				2
#define OUTSIDE				3

/* Global Variables */
BookNode *node = NULL;
int *book_hash_table = NULL;
int hash_table_size = 0;
int num_allnode = 0;
int num_usednode = 0;
int num_allalternative = 0;
int num_usedalternative = 0;
int num_history = 0;
char status[80];

/* ETC Tables */
int pow3[10] = { 1, 3, 9, 27, 81, 243, 729, 2187, 6561, 19683 };
int line_hash[2][8][6561];
int row_no[100];
int row_index[100];
int col_no[100];
int col_index[100];
int flip8[6561];
int *symmetry_map[8];
int *inv_symmetry_map[8];
int b1_b1_map[100], g1_b1_map[100], g8_b1_map[100], b8_b1_map[100];
int a2_b1_map[100], a7_b1_map[100], h7_b1_map[100], h2_b1_map[100];

/* Resources of myrandom */
#define TYPE_3				3		/* x**31 + x**3 + 1 */
#define SEP_3				3
#define	DEG_3				31
unsigned long my_randtbl[DEG_3 + 1] = { TYPE_3,
	0x9a319039U, 0x32d9c024U, 0x9b663182U, 0x5da1f342U, 
	0xde3b81e0U, 0xdf0a6fb5U, 0xf103bc02U, 0x48f340fbU, 
	0x7449e56bU, 0xbeb1dbb0U, 0xab5c5918U, 0x946554fdU, 
	0x8c2e680fU, 0xeb3d799fU, 0xb11ee0b7U, 0x2d436b86U, 
	0xda672e2aU, 0x1588ca88U, 0xe369735dU, 0x904f35f7U, 
	0xd7158fd6U, 0x6fa6f051U, 0x616e6b96U, 0xac94efdcU, 
	0x36413f93U, 0xc622c298U, 0xf5a42ab8U, 0x8a88d77bU, 
	0xf5ad9d0eU, 0x8999220bU, 0x27fb47b9U };
long *my_state = (long *) &my_randtbl[ 1 ];
long *my_fptr = (long *) &my_randtbl[ SEP_3 + 1 ];
long *my_rptr = (long *) &my_randtbl[ 1 ];
long *my_end_ptr = (long *) &my_randtbl[ DEG_3 + 1 ];

/********************************/
/* Functions derived from Zebra */
/********************************/
long my_random(void)
{
	/*********************************************/
	/* original code is my_random() (myrandom.c) */
	/*********************************************/
	long i;
	
	*my_fptr += *my_rptr;
	i = (*my_fptr >> 1)&0x7fffffff; /* chucking least random bit */
	if (++my_fptr >= my_end_ptr ) {
		my_fptr = my_state;
		++my_rptr;
	} else {
		if (++my_rptr >= my_end_ptr) {
			my_rptr = my_state;
		}
	}
	return i;
}

int my_srandom(int x)
{
	/**********************************************/
	/* original code is my_srandom() (myrandom.c) */
	/**********************************************/
	int i, j;

	j = 1;
	my_state[ 0 ] = x;
	for (i = 1; i < DEG_3; i++) {
		my_state[i] = 1103515245*my_state[i - 1] + 12345;
	}
	my_fptr = &my_state[SEP_3];
	my_rptr = &my_state[0];
	for(i = 0; i < 10*DEG_3; i++) {
		my_random();
	}
	return 0;
}

void init()
{
	int i,j,k;
	int pos;
	int row[10];

	/********************************************/
	/* original code is init_maps() (osfbook.c) */
	/********************************************/
	for (i = 1; i <= 8; i++) {
		for (j = 1; j <= 8; j++) {
			pos = 10 * i + j;
			b1_b1_map[pos] = pos;
			g1_b1_map[pos] = 10 * i + (9 - j);
			g8_b1_map[pos] = 10 * (9 - i) + (9 - j);
			b8_b1_map[pos] = 10 * (9 - i) + j;
			a2_b1_map[pos] = 10 * j + i;
			a7_b1_map[pos] = 10 * j + (9 - i);
			h7_b1_map[pos] = 10 * (9 - j) + (9 - i);
			h2_b1_map[pos] = 10 * (9 - j) + i;
		}
	}
	symmetry_map[0] = b1_b1_map;
	inv_symmetry_map[0] = b1_b1_map;

	symmetry_map[1] = g1_b1_map;
	inv_symmetry_map[1] = g1_b1_map;

	symmetry_map[2] = g8_b1_map;
	inv_symmetry_map[2] = g8_b1_map;

	symmetry_map[3] = b8_b1_map;
	inv_symmetry_map[3] = b8_b1_map;

	symmetry_map[4] = a2_b1_map;
	inv_symmetry_map[4] = a2_b1_map;

	symmetry_map[5] = a7_b1_map;
	inv_symmetry_map[5] = h2_b1_map; 

	symmetry_map[6] = h7_b1_map;
	inv_symmetry_map[6] = h7_b1_map;

	symmetry_map[7] = h2_b1_map;
	inv_symmetry_map[7] = a7_b1_map;

	for (i = 0; i < 8; i++) {
		symmetry_map[i][NULL_MOVE] = NULL_MOVE;
	}
	/*************************************************/
	/* original code is init_patterns() (patterns.c) */
	/*************************************************/
	for (i = 1; i <= 8; i++) {
		for (j = 1; j <= 8; j++) {
			pos = 10 * i + j;
			row_no[pos] = i - 1;
			row_index[pos] = j - 1;
			col_no[pos] = j - 1;
			col_index[pos] = i - 1;
		}
	}
	/********************************************************/
	/* original code is transformation_setup() (patterns.c) */
	/********************************************************/
	for (i = 0; i < 8; i++) {
		row[i] = 0;
	}
	for (i = 0; i < 6561; i++) {
		flip8[i] = 0;
		for (j = 0; j < 8; j++) {
			flip8[i] += row[j] * pow3[7 - j];
		}
		j = 0;
		do {
			row[j]++;
			if (row[j] == 3) {
				row[j] = 0;
			}
			j++;
		} while ((row[j - 1] == 0) && (j < 8));
	}
	/************************************************/
	/* original code is  prepare_hash() (osfbook.c) */
	/************************************************/
	my_srandom(0);
	for (i = 0; i < 2; i++) {
		for (j = 0; j < 8; j++) {
			for (k = 0; k < 6561; k++) {
				if (my_random() % 2) {
					line_hash[i][j][k] = my_random();
				} else {
					line_hash[i][j][k] = -my_random();
				}
			}
		}
	}
	hash_table_size = 0;           
}

int allalloc(int size)
{
	int new_size,new_memory,i;

	/*************************************************/
	/* original code is set_allocation() (osfbook.c) */
	/*************************************************/
	node = (BookNode *) malloc(size * sizeof( BookNode ));
	if (node == NULL) {
		return -1;
	}
	/*****************************************************/
	/* original code is rebuild_hash_table() (osfbook.c) */
	/*****************************************************/
	new_size = 2 * size;
	new_memory = new_size * sizeof(int);
	book_hash_table = (int *) malloc(new_memory);
	if (book_hash_table == NULL) {
		return -2;
	}
	hash_table_size = new_size;
	/********************************************************/
	/* original code is create_hash_reference() (osfbook.c) */
	/********************************************************/
	for (i = 0; i < hash_table_size; i++) {
		book_hash_table[i] = EMPTY_HASH_SLOT;
	}
	return 0;
}

void allfree()
{
	free(node);
	free(book_hash_table);
}

int searchnode(unsigned char *board,int *orientation)
{
	/**************************************************************/
	/* original code is get_hash(),probe_hash_table() (osfbook.c) */
	/**************************************************************/
	static int maskdef[3] = {1,0,2};
	int row_pattern[8];
	int col_pattern[8];
	int i, j;
	int min_map;
  	int min_hash0, min_hash1;
  	int out[8][2];
	int mask;
	int pos;
	int val1,val2;
	int slot;

	/*********************************************************/
	/* original code is compute_line_patterns() (patterns.c) */
	/*********************************************************/
	for (i = 0; i < 8; i++) {
		row_pattern[i] = 0;
		col_pattern[i] = 0;
	}
	for (i = 1; i <= 8; i++) {
		for (j = 1; j <= 8; j++) {
			pos = 10 * i + j;
			mask = maskdef[board[pos]];
			row_pattern[row_no[pos]] += mask * pow3[row_index[pos]];
			col_pattern[col_no[pos]] += mask * pow3[col_index[pos]];
		}
	}
	/********************************************/
	/* original code is get_hash() (osfbook.c)  */
	/********************************************/
  	for (i = 0; i < 8; i++) {
		for (j = 0; j < 2; j++) {
			out[i][j] = 0;
		}
	}
	for (i = 0; i < 8; i++) {
		/* b1 -> b1 */      
		out[0][0] ^= line_hash[0][i][row_pattern[i]];
		out[0][1] ^= line_hash[1][i][row_pattern[i]];
		/* g1 -> b1 */
		out[1][0] ^= line_hash[0][i][flip8[row_pattern[i]]];
		out[1][1] ^= line_hash[1][i][flip8[row_pattern[i]]];
		/* g8 -> b1 */
		out[2][0] ^= line_hash[0][i][flip8[row_pattern[7 - i]]];
		out[2][1] ^= line_hash[1][i][flip8[row_pattern[7 - i]]];
		/* b8 -> b1 */
		out[3][0] ^= line_hash[0][i][row_pattern[7 - i]];
		out[3][1] ^= line_hash[1][i][row_pattern[7 - i]];
		/* a2 -> b1 */
		out[4][0] ^= line_hash[0][i][col_pattern[i]];
		out[4][1] ^= line_hash[1][i][col_pattern[i]];
		/* a7 -> b1 */
		out[5][0] ^= line_hash[0][i][flip8[col_pattern[i]]];
		out[5][1] ^= line_hash[1][i][flip8[col_pattern[i]]];
		/* h7 -> b1 */
		out[6][0] ^= line_hash[0][i][flip8[col_pattern[7 - i]]];
		out[6][1] ^= line_hash[1][i][flip8[col_pattern[7 - i]]];
		/* h2 -> b1 */
		out[7][0] ^= line_hash[0][i][col_pattern[7 - i]];
		out[7][1] ^= line_hash[1][i][col_pattern[7 - i]];
	}
	min_map = 0;
	min_hash0 = out[0][0];
	min_hash1 = out[0][1];
	for (i = 1; i < 8; i++) {
		if ( (out[i][0] < min_hash0) || ((out[i][0] == min_hash0) && (out[i][1] < min_hash1)) ) {
			min_map = i;
			min_hash0 = out[i][0];
			min_hash1 = out[i][1];
		}
	}
	val1 = abs( min_hash0 );
	val2 = abs( min_hash1 );
	*orientation = min_map;
	/***************************************************/
	/* original code is probe_hash_table() (osfbook.c) */
	/***************************************************/
	slot = val1 % hash_table_size;
	while ((book_hash_table[slot] != EMPTY_HASH_SLOT) && (node[book_hash_table[slot]].hash_val2 != val2 || node[book_hash_table[slot]].hash_val1 != val1) ) {
		slot = (slot + 1) % hash_table_size;
	}
	return book_hash_table[slot];
}

int loadnodes(char *fn)
{
	/*******************************************************/
	/* original code is read_binary_database() (osfbook.c) */
	/*******************************************************/
	FILE *fp;
	unsigned short magic1,magic2;
	int nodecount;
	int i;
	int gotcount;
	int slot;

	/* Open File */
	fp = fopen(fn,"rb");
	if (!fp) {
		goto openerror;
	}
	/* Check Magic Code */
	if (fread(&magic1,sizeof(unsigned short),1,fp) != 1) {
		goto readerror;
	}
	if (magic1 != 2718) {
		goto magicerror;
	}
	if (fread(&magic2,sizeof(unsigned short),1,fp) != 1) {
		goto magicerror;
	}
	if (magic2 != 2818) {
		goto magicerror;
	}
	/* Get Node Count */
	if (fread(&nodecount,sizeof(int),1,fp) != 1) {
		goto readerror;
	}
	/* Allocate Memory */
	if (allalloc(nodecount + NODE_TABLE_SLACK ) < 0) {
		goto allocerror;
	}
	/* Get All Node */
	gotcount = 0;
	num_allalternative = 0;
	for (i = 0; i < nodecount; i++) {
		if (fread(&node[i].hash_val1,sizeof( int ),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].hash_val2,sizeof( int ),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].black_minimax_score,sizeof(short),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].white_minimax_score,sizeof(short),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].best_alternative_move,sizeof(short),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].alternative_score,sizeof( short ),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		if (fread(&node[i].flags,sizeof(unsigned short),1,fp) != 1) {
			goto end;	/* to use broken file */
		}
		node[i].flags &= ~USEDFLG;	/* only for ZebraBookConv */
		if (node[i].alternative_score != NO_SCORE) {
			num_allalternative += 1;
		}
		gotcount += 1;
	}
end:
	num_allnode = gotcount;	/* copy to global variable to use later */
	fclose(fp);
	/********************************************************/
	/* original code is create_hash_reference() (osfbook.c) */
	/********************************************************/
	for (i = 0; i < nodecount; i++) {
		/***************************************************/
		/* original code is select_hash_slot() (osfbook.c) */
		/***************************************************/
		slot = node[i].hash_val1 % hash_table_size;
		while ( book_hash_table[slot] != EMPTY_HASH_SLOT ) {
			slot = (slot + 1) % hash_table_size;
		}
		book_hash_table[slot] = i;
	}
	return nodecount;
openerror:
	return -1;
readerror:
	fclose(fp);
	return -2;
magicerror:
	fclose(fp);
	return -3;
allocerror:
	fclose(fp);
	return -4;
}

/************************************/
/* Functions ZebraBookConv original */
/************************************/
int getcandidate(unsigned char *board,unsigned char turn,unsigned char *cand)
{
	static unsigned char opp[3] = {WHITESQ,EMPTY,BLACKSQ};
	static int addval[8] = {(-1*10)+0,(-1*10)+1,(0*10)+1,(1*10)+1,(1*10)+0,(1*10)-1,(0*10)-1,(-1*10)-1};
	int dir,x,y,i,pos,pos1;
	int candnum;
	int changenum;

	candnum = 0;
	for (x = 1; x <= 8; x++) {
		for (y = 1; y <= 8; y++) {
			pos = 10*y+x;
			if (board[pos] != EMPTY) {
				continue;
			}
			for (dir = 0; dir < 8; dir++) {
				pos1 = pos;
				changenum = 0;
				for (i = 0; i < 8; i++) {
					pos1 += addval[dir];
					if (board[pos1] != opp[turn]) {
						break;
					}
					changenum += 1;
				}
				if (board[pos1] == turn) {
					if (changenum > 0) {
						cand[candnum++] = pos;
						break;
					}
				}
			}
		}
	}
	return candnum;
}

void setstone(unsigned char *inboard,unsigned char turn,unsigned char pos,unsigned char *board)
{
	static unsigned char opp[3] = {WHITESQ,EMPTY,BLACKSQ};
	static int addval[8] = {(-1*10)+0,(-1*10)+1,(0*10)+1,(1*10)+1,(1*10)+0,(1*10)-1,(0*10)-1,(-1*10)-1};
	int dir,i,pos1;
	int changenum;

	for (i = 0; i < 100; i++) {
		board[i] = inboard[i];
	}
	for (dir = 0; dir < 8; dir++) {
		pos1 = pos;
		changenum = 0;
		for (i = 0; i < 8; i++) {
			pos1 += addval[dir];
			if (board[pos1] != opp[turn]) {
				break;
			}
			changenum += 1;
		}
		if (board[pos1] == turn) {
			for (i = 0; i < changenum; i++) {
				pos1 -= addval[dir];
				board[pos1] = turn;
			}
		}
	}
	board[pos] = turn;
}

void outputhistory(FILE *fp,unsigned char *history,int deep,int zebrascore)
{
	int i,x,y;

	/**********************/
	/* Check History Data */
	/**********************/
	if (deep > 60) {
		/* Illegal Data */
		return;
	}
	for (i = 0; i <= deep; i++) {
		y = history[i]/10;
		x = history[i]%10;
		if (y < 1 || y > 8 || x < 1 || x > 8) {
			/* Illegal Data */
			return;
		}
	}
	/******************/
	/* Output History */
	/******************/
	for (i = 0; i <= deep; i++) {
		fprintf(fp,"%c%c",'A'+(history[i]%10)-1,'1'+(history[i]/10-1));
	}
	fprintf(fp," ; %.3f\n",(double)zebrascore/128.0);
	num_history += 1;
}

void printstatus(char *status)
{
	int i;
	char statusline[80];

	/********************/
	/* Clear Old Status */
	/********************/
	for (i = 0; i < 80; i++) {
		statusline[i] = ' ';
	}
	statusline[79] = 0;
	printf("%c%s",0x0d,statusline);
	/********************/
	/* Print New Status */
	/********************/
	for (i = 0; i < 80; i++) {
		statusline[i] = status[i];
	}
	statusline[79] = 0;
	printf("%c%s",0x0d,statusline);
}

int outputtree(FILE *fp,unsigned char *board,unsigned char turn,int deep,int passflg,unsigned char *history)
{
	static unsigned char opp[3] = {WHITESQ,EMPTY,BLACKSQ};
	static int lastpercent = 0;
	int i;
	unsigned char cand[64];
	int candnum;
	unsigned char newboard[100];
	int nodeindex;
	int output;
	int childnum;
	int score;
	int orientation;
	double percent;
	int ipercent;

	/*************************/
	/* Search node from book */
	/*************************/
	nodeindex = searchnode(board,&orientation);
	if (nodeindex < 0) {
		/* Node Not Exist */
		return -1;
	}
	/**************/
	/* Make score */
	/**************/
	score = node[nodeindex].black_minimax_score;
	if (score >= CONFIRMED_WIN) {
		score -= CONFIRMED_WIN;
		score *= 128;
	} else if (score <= -CONFIRMED_WIN) {
		score += CONFIRMED_WIN;
		score *= 128;
	}
	/********************************/
	/* Check already visited or not */
	/********************************/
	if (node[nodeindex].flags & USEDFLG) {
		/* Already visited, do not execute child search */
		outputhistory(fp,history,deep-1,score);
		return 1;	/* Node Exist */
	}
	node[nodeindex].flags |= USEDFLG;
	num_usednode += 1;
	percent = 100.0*num_usednode/num_allnode;
	ipercent = (int)percent;
	if (ipercent != lastpercent) {
		sprintf(status,"Converting .. %d%% done",ipercent);
		printstatus(status);
		lastpercent = ipercent;
	}
	/*****************************************/
	/* Get candidate And check pass/gameover */
	/*****************************************/
	candnum = getcandidate(board,turn,cand);
	if (candnum == 0) {
		/* Case of pass */
		if (passflg) {
			/* Case of gameover */
			outputhistory(fp,history,deep-1,score);
			return 1;	/* Node Exist */
		}
		return outputtree(fp,board,opp[turn],deep,1,history);
	}
	/************************/
	/* Execute Child Search */
	/************************/
	childnum = 0;
	for (i = 0; i < candnum; i++) {
		setstone(board,turn,cand[i],newboard);
		history[deep] = cand[i];
		if (outputtree(fp,newboard,opp[turn],deep+1,0,history) > 0) {
			childnum += 1;
		}
	}
	/**********************************/
	/* Output alternative move or not */
	/**********************************/
	if (node[nodeindex].alternative_score != NO_SCORE) {
		if (node[nodeindex].flags & FULL_SOLVED) {
			/* When Ful lSolved Node, not output */
			output = 0;
		} else if (node[nodeindex].flags & WLD_SOLVED) {
			/* When WLD Solved Node, output only when WLD not change */
			output = 0;
			if (score > 0 && node[nodeindex].alternative_score > 0) {
				output = 1;
			}
			if (score < 0 && node[nodeindex].alternative_score < 0) {
				output = 1;
			}
			if (score == 0 && node[nodeindex].alternative_score == 0) {
				output = 1;
			}
		} else {
			/* When Mid Node, output always */
			output = 1;
		}
		if (output) {
			history[deep] = inv_symmetry_map[orientation][node[nodeindex].best_alternative_move];
			outputhistory(fp,history,deep,node[nodeindex].alternative_score);
			num_usedalternative += 1;
		}
	}
	/*************************/
	/* Output history or not */
	/*************************/
	if (childnum == 0) {
		outputhistory(fp,history,deep-1,score);
	}
	/*********************************/
	/* Return with node exist status */
	/*********************************/
	return 1;
}

int main(int argc, char *argv[])
{
	int nodenum;
	FILE *outfp;
	unsigned char history[64];
	unsigned char board[100];
	int x,y;
	char status[32768];
	int overwrite;
	int retval;

	printf("************************************************************\n");
	printf("* Zebra Book Converter Ver1.00                             *\n");
	printf("* (c) 2008 T.Ishii (t-ishii@js2.so-net.ne.jp)              *\n");
	printf("*                                                          *\n");
	printf("*   Author of Zebra: Gunnar Andersson (gunnar@radagast.se) *\n");
	printf("************************************************************\n");
	printf("- Notice -\n");
	printf(" Zebra is a famous reversi program released under the GPL Version 2.\n");
	printf(" This program is made using Zebra source code.\n");
	printf(" So, this program is also released under the GPL Version 2.\n");
	printf("\n");
	printf(" You may use/modify/redistribute this program under the GPL Version 2.\n");
	printf(" There is no warranty. See the file COPYING for more information.\n");
	printf("\n");
	retval = -1;
	/* Get Parameters */
	overwrite = 0;
	if (argc == 4) {
		if (argv[3][0] != '-' || argv[3][1] != 'w' || argv[3][2] != 0) {
			goto usage;
		}
		overwrite = 1;
	} else if (argc != 3) {
		goto usage;
	}
	/* Check Output Path */
	outfp = fopen(argv[2],"r");
	if (outfp) {
		fclose(outfp);
		if (!overwrite) {
			goto outputexisterror;
		}
	}
	outfp = fopen(argv[2],"w");
	if (!outfp) {
		goto outputerror;
	}
	/* Initialize */
	init();
	/* Load book */
	sprintf(status,"Loading %s",argv[1]);
	printstatus(status);
	nodenum = loadnodes(argv[1]);
	if (nodenum < 0) {
		goto inputerror;
	}
	sprintf(status,"Source: %s (%d nodes + %d alternatives)",argv[1],num_allnode,num_allalternative);
	printstatus(status);
	printf("\n");
	/* Make root board */
	for (x = 0; x <= 9; x++) {
		for (y = 0; y <= 9; y++) {
			if (x == 0 || x == 9 || y == 0 || y == 9) {
				board[10*y+x] = OUTSIDE;
			} else {
				board[10*y+x] = EMPTY;
			}
		}
	}
	board[4*10+4] = WHITESQ;
	board[4*10+5] = BLACKSQ;
	board[5*10+4] = BLACKSQ;
	board[5*10+5] = WHITESQ;
	/* Execute convert */
	sprintf(status,"Output: %s",argv[2]);
	printstatus(status);
	printf("\n\n");
	outputtree(outfp,board,BLACKSQ,0,0,history);
	fclose(outfp);
	sprintf(status,"Convert is successfully done.");
	printstatus(status);
	printf("\n\n");
	printf("- Convert Information -\n");
	printf("used node:        %d\n",num_usednode);
	printf("used alternative: %d\n",num_usedalternative);
	printf("output history:   %d\n",num_history);
	return 0;
/********************************************/
/* Error Return
/********************************************/
outputexisterror:
	printf("\nError: Output file exist.\n");
	retval = -2;
	goto usage;
outputerror:
	printf("\nError: Can't open %s.\n",argv[2]);
	retval = -2;
	goto usage;
inputerror:
	switch (nodenum) {
	case -1:
		/* Open Error */
		printf("\nError: Can't open %s.\n",argv[1]);
		break;
	case -2:
		/* Read Error */
		printf("\nError: Can't read %s.\n",argv[1]);
		break;
	case -3:
		/* Format Error */
		printf("\nError: Source file format is illegal.\n");
		break;
	default:
		/* Memory Allocation Error */
		printf("\nError: Memory allocation error.\n");
	}
	retval = -3;
	goto usage;
usage:
	printf("\nUsage: ZebraBookConv SRCPATH OUTPATH [-w]\n");
	printf("\n");
	printf("SRCPATH: Path of Zebra Book File for Read.\n");
	printf("OUTPATH: Path of History Text File for Write. \n");
	printf("\n");
	printf("When file exist in OUTPATH, command will fail unless -w specified.\n");  
	return retval;
}

